#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <camel/camel-i18n.h>
#include <camel/camel-provider.h>
#include <camel/camel-session.h>
#include <camel/camel-url.h>
#include <camel/camel-service.h>
#include "camel-ogo-store.h"

static void add_hash (guint *hash, char *s);
static guint ogo_url_hash (gconstpointer key);
static gint check_equal (char *s1, char *s2);
static gint ogo_url_equal (gconstpointer a, gconstpointer b);


CamelProviderConfEntry ogo_conf_entries[] = {
	{ CAMEL_PROVIDER_CONF_END }
};


static CamelProvider ogo_provider = {
	"ogo",
	N_("OpenGroupware.org"),

	N_("For accessing OpenGroupware.org servers"),

	"mail",

	CAMEL_PROVIDER_IS_REMOTE | CAMEL_PROVIDER_IS_SOURCE |
	CAMEL_PROVIDER_SUPPORTS_SSL,

	CAMEL_URL_NEED_USER | CAMEL_URL_NEED_HOST | CAMEL_URL_ALLOW_AUTH,

	ogo_conf_entries,

	/* ... */
};

CamelServiceAuthType camel_ogo_password_authtype = {
	N_("Password"),
	
	N_("This option will connect to the IMAP server using a "
	   "plaintext password."),
	
	"",
	TRUE
};

static int
ogo_auto_detect_cb (CamelURL *url, GHashTable **auto_detected,
			 CamelException *ex)
{
	*auto_detected = g_hash_table_new (g_str_hash, g_str_equal);

	g_hash_table_insert (*auto_detected, g_strdup ("poa"),
			     g_strdup (url->host));

	return 0;
}

void
camel_provider_module_init(void)
{
	CamelException ex = CAMEL_EXCEPTION_INITIALISER;

	ogo_provider.url_hash = ogo_url_hash;
	ogo_provider.url_equal = ogo_url_equal;
	ogo_provider.auto_detect = ogo_auto_detect_cb;
	ogo_provider.authtypes = g_list_prepend (ogo_provider.authtypes, &camel_ogo_password_authtype);

	ogo_provider.object_types[CAMEL_PROVIDER_STORE] = camel_ogo_store_get_type ();
	camel_provider_register(&ogo_provider);
}


static void
add_hash (guint *hash, char *s)
{
	if (s)
		*hash ^= g_str_hash(s);
}

static guint
ogo_url_hash (gconstpointer key)
{
	const CamelURL *u = (CamelURL *)key;
	guint hash = 0;

	add_hash (&hash, u->user);
	add_hash (&hash, u->authmech);
	add_hash (&hash, u->host);
	hash ^= u->port;
	
	return hash;
}

static gint
check_equal (char *s1, char *s2)
{
	if (s1 == NULL) {
		if (s2 == NULL)
			return TRUE;
		else
			return FALSE;
	}
	
	if (s2 == NULL)
		return FALSE;

	return strcmp (s1, s2) == 0;
}

static gint
ogo_url_equal (gconstpointer a, gconstpointer b)
{
	const CamelURL *u1 = a, *u2 = b;
	
	return check_equal (u1->user, u2->user)
		&& check_equal (u1->authmech, u2->authmech)
		&& check_equal (u1->host, u2->host)
		&& u1->port == u2->port;
}
